package org.fouram.service;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.fouram.constants.WebConstants;
import org.fouram.core.base.service.BaseService;
import org.fouram.core.util.DateUtil;
import org.fouram.core.util.LoggerUtil;
import org.fouram.dto.output.SyncWeixinUserOutput;
import org.fouram.enums.DelFlagEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class SyncSapUserToWxService extends BaseService {

	@Autowired
	private WxUserService wxUserService;
	@Autowired
	private SapUserService sapUserService;
	@Autowired
	private WxDepartService wxDepartService;

	public SyncWeixinUserOutput syncWeixinUser() throws Exception {
		if (WebConstants.CACHE.containsKey(WebConstants.CACHE_SYNC_SAP)) {
			return SyncWeixinUserOutput.builder().state("同步中")
					.time(WebConstants.CACHE.get(WebConstants.CACHE_SYNC_SAP)).build();
		}
		ExecutorService threadPool = Executors.newSingleThreadExecutor();
		threadPool.execute(new Runnable() {
			@Override
			public void run() {
				try {
					LoggerUtil.info("同步SAP执行开始:" + DateUtil.getTime());
					taskBiz();
					LoggerUtil.info("同步SAP执行开始:" + DateUtil.getTime());
				} catch (Exception e) {
					LoggerUtil.error(e);
				}
			}
		});
		threadPool.shutdown();// 任务执行完毕，关闭线程池
		return SyncWeixinUserOutput.builder().state("正在同步，请稍等").build();
	}

	@Transactional
	public void taskBiz() throws Exception {
		WebConstants.CACHE.put(WebConstants.CACHE_SYNC_SAP, DateUtil.getTime());
		wxDepartService.saveWxDeparts();
		wxUserService.saveWxUsers();
		// 更新所有数据无效
		sapUserService.updateAllDelFlag(DelFlagEnum.UNABLE.getCode());
		// 保存SAP用户数据
		sapUserService.saveOrUpdateSapUsers();
		// 更新异常情况，敏感数据、入职等
		sapUserService.updateAbnormalResult();
		// 更新用户头像
		sapUserService.updateAvatars();
		// NORMAL数据更新同步企业微信
		sapUserService.updateAllNormalSyncWxResult();
		WebConstants.CACHE.remove(WebConstants.CACHE_SYNC_SAP);
	}
}
